==> csnlp-1.3.1.tar.gz: csnlp-1.3.1/src/csnlp/core/variables.py <==
from typing import Dict, Generic, Literal, Tuple, Type, TypeVar, Union

import casadi as cs

from csnlp.util.data import dict2struct, struct_symSX
from csnlp.util.funcs import cached_property, invalidate_cache

T = TypeVar("T", cs.SX, cs.MX)


class HasVariables(Generic[T]):
    """Class for creating and storing symbolic variables of an NLP problem."""

    def __init__(self, sym_type: Literal["SX", "MX"] = "SX") -> None:
        """Instantiate the class.

        Parameters
        ----------
        sym_type : "SX" or "MX", optional
            The CasADi symbolic variable type to use in the NLP, by default "SX".
        """
        self._csXX: Type[T] = getattr(cs, sym_type)
        self._vars: Dict[str, T] = {}
        self._x = self._csXX()

    @property
    def x(self) -> T:
        """Gets the primary variables of the NLP scheme in vector form."""
        return self._x

    @property
    def nx(self) -> int:
        """Number of variables in the NLP scheme."""
        return self._x.shape[0]

    @cached_property
    def variables(self) -> Union[struct_symSX, Dict[str, cs.MX]]:
        """Gets the primal variables of the NLP scheme."""
        # TODO: test this gets cleared despite the inheritance
        return dict2struct(self._vars)

    @invalidate_cache(variables)
    def variable(self, name: str, shape: Tuple[int, int] = (1, 1)) -> T:
        """
        Adds a variable to the NLP problem.

        Parameters
        ----------
        name : str
            Name of the new variable. Must not be already in use.
        shape : tuple[int, int], optional
            Shape of the new variable. By default, a scalar.

        Returns
        -------
        var : casadi.SX
            The symbol of the new variable.

        Raises
        ------
        ValueError
            Raises if there is already another variable with the same name.
        """
        if name in self._vars:
            raise ValueError(f"Variable name '{name}' already exists.")
        var = self._csXX.sym(name, *shape)
        self._vars[name] = var
        self._x = cs.vertcat(self._x, cs.vec(var))
        return var

==> csnlp-1.3.2.tar.gz: csnlp-1.3.2/src/csnlp/core/variables.py <==
from typing import Dict, Generic, Literal, Tuple, Type, TypeVar, Union

import casadi as cs

from csnlp.util.data import dict2struct, struct_symSX
from csnlp.util.funcs import cached_property, invalidate_cache

T = TypeVar("T", cs.SX, cs.MX)


class HasVariables(Generic[T]):
    """Class for creating and storing symbolic variables of an NLP problem."""

    def __init__(self, sym_type: Literal["SX", "MX"] = "SX") -> None:
        """Instantiate the class.

        Parameters
        ----------
        sym_type : "SX" or "MX", optional
            The CasADi symbolic variable type to use in the NLP, by default "SX".
        """
        self._csXX: Type[T] = getattr(cs, sym_type)
        self._vars: Dict[str, T] = {}
        self._x = self._csXX()

    @property
    def x(self) -> T:
        """Gets the primary variables of the NLP scheme in vector form."""
        return self._x

    @property
    def nx(self) -> int:
        """Number of variables in the NLP scheme."""
        return self._x.shape[0]

    @cached_property
    def variables(self) -> Union[struct_symSX, Dict[str, cs.MX]]:
        """Gets the primal variables of the NLP scheme."""
        return dict2struct(self._vars)

    @invalidate_cache(variables)
    def variable(self, name: str, shape: Tuple[int, int] = (1, 1)) -> T:
        """Adds a variable to the NLP problem.

        Parameters
        ----------
        name : str
            Name of the new variable. Must not be already in use.
        shape : tuple[int, int], optional
            Shape of the new variable. By default, a scalar.

        Returns
        -------
        var : casadi.SX
            The symbol of the new variable.

        Raises
        ------
        ValueError
            Raises if there is already another variable with the same name.
        """
        if name in self._vars:
            raise ValueError(f"Variable name '{name}' already exists.")
        var = self._csXX.sym(name, *shape)
        self._vars[name] = var
        self._x = cs.vertcat(self._x, cs.vec(var))
        return var
